/***************************************************************************
 *   Copyright (C) 2008 by Dario Freddi <drf@kdemod.ath.cx>                *
 *   Copyright (C) 2008 by Alessandro Diaferia <alediaferia@gmail.com>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "ConnectedDeviceDelegate.h"
#include "ConnectedDevicesModel.h"
#include "AbstractDevice.h"

#include <QPainter>
#include <QStyleOptionViewItemV4>
#include <QApplication>
#include <QFontMetrics>
#include <QLinearGradient>

#include <KIcon>
#include <KGlobalSettings>

const int ICON_SIZE = 64;
const int MARGIN = 5;
const int ITEM_TOP_MARGIN = 10;
const int ROW_HEIGHT = 100;

ConnectedDeviceDelegate::ConnectedDeviceDelegate(QObject *parent)
        : QStyledItemDelegate(parent), m_font(KGlobalSettings::generalFont())
{
}

ConnectedDeviceDelegate::~ConnectedDeviceDelegate()
{
}

void ConnectedDeviceDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QStyleOptionViewItemV4 opt(option);

    painter->fillRect(opt.rect, opt.palette.color(QPalette::Button));
//     QStyle *style = QApplication::style();
//     style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, painter, opt.widget);

    QPixmap pixmap(opt.rect.size());
    pixmap.fill(Qt::transparent);
    QPainter p(&pixmap);
    p.translate(-opt.rect.topLeft());
    p.setRenderHint(QPainter::Antialiasing);

    QRect clipRect(opt.rect);
    p.setClipRect(clipRect);

    // NOTE: maybe this code will be only used for operations overlay
//     if (opt.state & QStyle::State_MouseOver || opt.state & QStyle::State_Selected) {
//         p.save();
//         p.setBrush(QBrush(opt.palette.color(QPalette::Dark)));
//         p.setPen(Qt::transparent);
//         p.drawRoundedRect(opt.rect, 10.0, 20.0, Qt::RelativeSize);
//         p.restore();
//     }

    paintBorderLines(&p, opt);

    AbstractDevice *device = index.data(ConnectedDevicesModel::ReferenceRole).value<AbstractDevice*>();
    // let's draw the icon
    KIcon icon(device->icon());

    QIcon::Mode iconMode = (opt.state & QStyle::State_MouseOver) ? QIcon::Active : QIcon::Normal;

    QRect iconRect(opt.rect.topLeft(), QSize(ICON_SIZE, ICON_SIZE));
    const QRect iconPlaceRect(opt.rect.topLeft(), QSize(ROW_HEIGHT, ROW_HEIGHT));
    iconRect.moveCenter(QPoint(ICON_SIZE / 2, iconPlaceRect.center().y()));
    icon.paint(&p, iconRect, Qt::AlignCenter, iconMode);

    // device name drawing
    QColor foregroundColor = /*(opt.state.testFlag(QStyle::State_Selected) || opt.state.testFlag(QStyle::State_MouseOver)) ?
                             opt.palette.color(QPalette::HighlightedText) :*/ opt.palette.color(QPalette::Text);

    // TODO: use QFontMetrics in order to determine right text places and sizes
    p.setPen(foregroundColor);
    clipRect.setSize(QSize(opt.rect.width() - ICON_SIZE - MARGIN, ROW_HEIGHT / 3));
    clipRect.translate(ICON_SIZE + MARGIN, 0);
    p.setClipRect(clipRect);

    QFont font = m_font; // TODO: choose a good font for the device name
    font.setBold(true);
    font.setPointSize(font.pointSize() + 1);
    p.setFont(font);

    p.drawText(clipRect, Qt::AlignLeft | Qt::AlignVCenter, index.data(Qt::DisplayRole).toString());
//     p.drawText(clipRect, Qt::AlignLeft | Qt::AlignBottom, device->deviceInfo()->model);

    p.end();

    painter->drawPixmap(opt.rect.topLeft(), pixmap);
}

QSize ConnectedDeviceDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QFont font = m_font;
    font.setBold(true);
    font.setPointSize(font.pointSize() + 1);
    QFontMetrics fm(font);

    return QSize(ICON_SIZE + MARGIN + fm.width(index.data(Qt::DisplayRole).toString()), qMax(ROW_HEIGHT, fm.height()));
}

void ConnectedDeviceDelegate::paintBorderLines(QPainter *painter, const QStyleOptionViewItem &option) const
{
    const QRect contentsRect = option.rect;
    // top line
    QLinearGradient grad(contentsRect.topLeft(), contentsRect.topRight());
    grad.setColorAt(0.0, Qt::transparent);
    grad.setColorAt(0.5, option.palette.color(QPalette::Light));
    grad.setColorAt(1, Qt::transparent);
    
    painter->save();
    QRect lineRect(contentsRect);
    lineRect.setSize(QSize(lineRect.width(), 1));
    painter->fillRect(lineRect, grad);
    painter->restore();

    // bottom line
    grad = QLinearGradient(contentsRect.bottomLeft(), contentsRect.bottomRight());
    grad.setColorAt(0.0, Qt::transparent);
    grad.setColorAt(0.5, option.palette.color(QPalette::Dark));
    grad.setColorAt(1, Qt::transparent);
    
    painter->save();
    lineRect.translate(0, contentsRect.height() - 1);
    painter->fillRect(lineRect, grad);
    painter->restore();

}


#include "ConnectedDeviceDelegate.moc"
