/***************************************************************************
 *   Copyright (C) 2008 by Dario Freddi <drf@kdemod.ath.cx>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "PropertiesWidget.h"

#include "AbstractDevice.h"
#include "AbstractDeviceInterface.h"

#include <KAction>
#include <kmessagebox.h>

#include <QToolBar>

PropertiesWidget::PropertiesWidget(AbstractDevice *d, QWidget *parent)
        : QWidget(parent),
        m_device(d)
{
    if (!d->deviceInfo()) {
        return;
    }

    setupUi(this);

    nameLabel->setText(d->deviceInfo()->name);
    modelLabel->setText(d->deviceInfo()->model);
    manufacturerLabel->setText(d->deviceInfo()->manufacturer);
    idLabel->setText(d->deviceInfo()->id);
    fileLabel->setText(d->deviceInfo()->supported_filetypes);
    infoLabel->setText(d->deviceInfo()->additional_info);

    if (!d->deviceInfo()->battery_capacity && !d->deviceInfo()->battery_level) {
        batteryBar->setFormat(i18n("Not available"));
    } else {
        batteryBar->setValue((d->deviceInfo()->battery_level * 100) / d->deviceInfo()->battery_capacity);
    }

    QVBoxLayout *l = new QVBoxLayout();
    QToolBar *t = new QToolBar();

    t->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    t->setIconSize(QSize(32, 32));

    l->addWidget(t);

    QList<QAction*> list;

    KAction *dis = new KAction(KIcon("network-connect"), i18n("&Disconnect Device"), this);

    list.append(dis);

    connect(dis, SIGNAL(triggered()), this, SLOT(triggerDisconnect()));

    foreach(KAction *action, d->specialActions()) {
        list.append(action);
    }

    t->addActions(list);

    actionsBox->setLayout(l);
}

PropertiesWidget::~PropertiesWidget()
{
}

void PropertiesWidget::triggerDisconnect()
{
    if (KMessageBox::questionYesNo(this, i18n("Do you really want to disconnect the device?"),
                                   i18n("Disconnect Device")) == KMessageBox::Yes) {
        m_device->interface()->disconnectDevice(m_device);
        deleteLater();
    }
}

#include "PropertiesWidget.moc"
